/***************************************************************
 * Name:      Music_Organizer__Main.cpp
 * Purpose:   Code for Application Frame
 * Author:    Jackie Team (jackie.team@gmail.com)
 * Created:   2007-11-26
 * Copyright: Jackie Team (http://musicorganizer.free.fr)
 * License:
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#ifdef __BORLANDC__
#pragma hdrstop
#endif //__BORLANDC__

#include "Music_Organizer__Main.h"
#ifndef __WXMSW__
#include "images/icon.xpm"
#endif
//helper functions
enum wxbuildinfoformat
{
	short_f, long_f
};

wxString wxbuildinfo ( wxbuildinfoformat format )
{
	wxString wxbuild ( wxVERSION_STRING );

	if ( format == long_f )
	{
#if defined(__WXMSW__)
		wxbuild << _T ( "-Windows" );
#elif defined(__WXMAC__)
		wxbuild << _T ( "-Mac" );
#elif defined(__UNIX__)
		wxbuild << _T ( "-Linux" );
#endif

#if wxUSE_UNICODE
		wxbuild << _T ( "-Unicode build" );
#else
		wxbuild << _T ( "-ANSI build" );
#endif // wxUSE_UNICODE
	}
	return wxbuild;
}

BEGIN_EVENT_TABLE ( Music_Organizer__Frame, wxFrame )
	EVT_CLOSE ( Music_Organizer__Frame::OnClose )
	EVT_MENU ( idMenuQuit, Music_Organizer__Frame::OnQuit )
	EVT_MENU ( idMenuAbout, Music_Organizer__Frame::OnAbout )
	EVT_MENU ( idMenuHelp, Music_Organizer__Frame::OnHelp )
	EVT_MENU ( idMenuRefresh, Music_Organizer__Frame::onRefresh)
	EVT_MENU ( idMenuDel, Music_Organizer__Frame::onDel)
	EVT_MENU ( idMenuSel, Music_Organizer__Frame::onSel)
	EVT_MENU ( idMenuOp, Music_Organizer__Frame::onOp)
	EVT_MENU ( ID_MSG, Music_Organizer__Frame::changeMenu)
	EVT_KEY_UP (Music_Organizer__Frame::onKeyUp)
END_EVENT_TABLE()




Music_Organizer__Frame::Music_Organizer__Frame ( wxFrame *frame, const wxString& title )
		: wxFrame ( frame, -1, title )
{
        wxString rcFile = wxGetHomeDir();
        rcFile << wxString::Format(_T("%c.morc"),SEPARATOR);
        OptionsHandler::loadFile(rcFile);
        
        m_locale.Init( wxLANGUAGE_FRENCH, wxLOCALE_CONV_ENCODING);
        wxLocale::AddCatalogLookupPathPrefix(wxT("./po"));
        m_locale.AddCatalog(wxT("musicorganizer"));

#if wxUSE_MENUS
	// create a menu bar
	wxMenuBar* mbar = new wxMenuBar();
	wxMenu* fileMenu = new wxMenu ( _T ( "" ) );
	fileMenu->Append ( idMenuQuit, _ ( "&Quit\tAlt-F4" ), _ ( "Exit application" ) );
	mbar->Append ( fileMenu, _ ( "&File" ) );

	editMenu = new wxMenu ( _T ( "" ) );
	editMenu->Append ( idMenuRefresh, _ ( "&Refresh\tF5" ), _ ( "Refresh the list" ) );
	editMenu->Append ( idMenuDel, _ ( "&Delete\tDel" ), _ ( "Delete selected line(s)" ) );
	editMenu->Append ( idMenuSel, _ ( "&Select all\tCtrl-A" ), _ ( "Select all items" ) );
	editMenu->Enable(idMenuDel,false);
	editMenu->Enable(idMenuSel,false);
	editMenu->AppendSeparator();
	editMenu->Append ( idMenuOp, _ ( "&Options" ), _ ( "Custom music organizer" ) );
	mbar->Append ( editMenu, _ ( "&Edit" ) );

	wxMenu* helpMenu = new wxMenu ( _T ( "" ) );
	helpMenu->Append ( idMenuHelp, _ ( "&Aide\tF1" ), _ ( "Online help" ) );
	helpMenu->Append ( idMenuAbout, _ ( "About" ), _ ( "About" ) );
	mbar->Append ( helpMenu, _ ( "&?" ) );

	SetMenuBar ( mbar );
#endif // wxUSE_MENUS

	// create a status bar with some information about the used wxWidgets version
	CreateStatusBar ( 1 );
	SetStatusText ( _ ( "Welcome!" ),0 );
	wxImage* splash = new wxImage();
	splash->AddHandler ( new wxPNGHandler() );
	splash->LoadFile ( _T ( "splashscreen.png" ) );
	wxSplashScreen* spl=new wxSplashScreen ( wxBitmap ( *splash ),wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_NO_TIMEOUT , 0,NULL,wxID_ANY,
	        wxDefaultPosition, wxDefaultSize,wxSIMPLE_BORDER|wxSTAY_ON_TOP );
#ifndef __WXMSW__
	wxIcon icone=wxIcon ( icon_xpm );
#else
	wxIcon icone=wxIcon(_T("A"));//A is the name in the .rc file
#endif
	SetIcon(icone);
	SetMinSize(wxSize(720,500));
	wxSplitterWindow* sp=new wxSplitterWindow ( this,wxSP_LIVE_UPDATE );
	panel3=new wxPanel ( this );
	tabControl=new MonBook ( panel3 );
	panel2=new ListCtrlPanel ( sp,tabControl );
	tabControl->setListe ( panel2->getMListe() );
	panel1=new wxPanel ( sp );
	arbo=new MonArbo ( panel1,panel2->getMListe(),tabControl );
	tabControl->setArbo ( arbo );
        panel2->getMListe()->setArbo(arbo);

	wxBoxSizer* sizer= new wxBoxSizer ( wxVERTICAL );
	wxBoxSizer* sizerArbo = new wxBoxSizer ( wxHORIZONTAL );
	wxBoxSizer* sizerTab = new wxBoxSizer ( wxHORIZONTAL );

	sizerTab->Add ( tabControl,1, wxEXPAND|wxALL );
	panel3->SetSizer ( sizerTab );
	sizerArbo->Add ( arbo,1, wxEXPAND|wxALL );
	panel1->SetSizer ( sizerArbo );
	sp->SplitVertically ( panel1,panel2,250 );
	sizer->Add ( sp,1,wxEXPAND|wxALL );
	sizer->Add ( panel3,0,wxEXPAND|wxALL );
	SetSizer ( sizer );

	SetAutoLayout ( true );
	spl->Destroy();


}


Music_Organizer__Frame::~Music_Organizer__Frame()
{
}

void Music_Organizer__Frame::OnClose ( wxCloseEvent &event )
{
	Destroy();
}

void Music_Organizer__Frame::OnQuit ( wxCommandEvent &event )
{
	Destroy();
}

void Music_Organizer__Frame::OnAbout ( wxCommandEvent &event )
{
	// First we create and fill the info object
	wxAboutDialogInfo info;
	info.SetName ( _T ( "Music Organizer" ) );
	info.SetVersion ( _T ( "2.0.0" ) );
	info.SetCopyright ( _T ( "(C) 2007 JackyTeam" ) );
	info.SetDescription ( _ ( "\"Music Organizer\" is an musical files' editor. \nFor example it can change tags, organize some files in directory\n and rename them." ) );
	info.SetWebSite ( _T ( "http://musicorganizer.free.fr" ), _ ( "Official website" ) );
	info.AddDeveloper ( _T ( "C. Gallet" ) );
	info.AddDeveloper ( _T ( "J. Charles" ) );
	info.AddDeveloper ( wxT ( "T. Tournié" ) );
	info.AddDeveloper ( _T ( "V. Petit" ) );
	info.SetLicense ( _T ( "http://www.gnu.org/licenses/gpl.html" ) );
	// Then we call wx.AboutBox giving it that info object
	wxAboutBox ( info );

}

void Music_Organizer__Frame::OnHelp ( wxCommandEvent &event )
{
	wxFrame* hlnkFrame=new wxFrame ( this,wxID_ANY,_ ( "Online help" ),wxDefaultPosition,wxDefaultSize, ( wxDEFAULT_FRAME_STYLE & ~ ( wxRESIZE_BORDER | wxRESIZE_BOX | wxMAXIMIZE_BOX ) ) | wxFRAME_FLOAT_ON_PARENT );
	wxPanel* hlnkPanel=new wxPanel ( hlnkFrame );
	wxHyperlinkCtrl* lnkHelp=new wxHyperlinkCtrl ( hlnkFrame,wxID_ANY,_T ( "http://musicorganizer.free.fr/Aide.html" ),_T ( "http://musicorganizer.free.fr/Aide.html" ) );
	wxBoxSizer* hlnkSizer=new wxBoxSizer ( wxHORIZONTAL );
	hlnkSizer->Add ( lnkHelp );
	hlnkPanel->SetSizer ( hlnkSizer );
	hlnkSizer->Fit(hlnkFrame);

	hlnkFrame->SetAutoLayout ( true );

	hlnkFrame->Center();
	hlnkFrame->Show();
}

void Music_Organizer__Frame::onRefresh(wxCommandEvent& event){
       wxCommandEvent e( wxEVT_COMMAND_MENU_SELECTED, ID_REFRESH);
       wxPostEvent(panel2->getMListe(), e );
}

void Music_Organizer__Frame::onDel(wxCommandEvent& event){
       wxCommandEvent e( wxEVT_COMMAND_MENU_SELECTED, ID_DEL);
       wxPostEvent(panel2->getMListe(), e );
}


void Music_Organizer__Frame::onSel(wxCommandEvent& event){
       wxCommandEvent e( wxEVT_COMMAND_MENU_SELECTED, ID_SEL);
       wxPostEvent(panel2->getMListe(), e );
}

void Music_Organizer__Frame::onOp(wxCommandEvent& event){
   opd = new OptionsDialog(this);
   opd->Center();
   opd->Show();
}

void Music_Organizer__Frame::changeMenu(wxCommandEvent& event){
   int op=event.GetInt();
   if(op==-1){//on desactive les deux items
	editMenu->Enable(idMenuDel,false);
	editMenu->Enable(idMenuSel,false);
   }else if(op==0){//on active les deux
	editMenu->Enable(idMenuDel,true);
	editMenu->Enable(idMenuSel,true);
   }else if(op==1){//on active uniquement le 1er
	editMenu->Enable(idMenuDel,true);
	editMenu->Enable(idMenuSel,false);
   }else if(op==2){//on active uniquement le 2nd
	editMenu->Enable(idMenuDel,false);
	editMenu->Enable(idMenuSel,true);
   }
}

void Music_Organizer__Frame::onKeyUp(wxKeyEvent& e){
   panel2->getMListe()->unblockKeyEvent();
   e.Skip();
}
