#ifndef COMPONENTS_H_INCLUDED
#define COMPONENTS_H_INCLUDED

/***************************************************************
 * Name:      compents.h
 * Purpose:   Defines Main Panel and Componants Classes
 * Author:    Jackie Team (jackie.team@gmail.com)
 * Created:   2007-12-07
 * Copyright: Jackie Team (http://musicorganizer.free.fr)
 * License: GNU GPL
 **************************************************************/
#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif

#include <wx/listctrl.h>
#include <wx/panel.h>
#include <wx/notebook.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/combobox.h>
#include <wx/arrstr.h>
#include <wx/bmpbuttn.h>
#include <wx/bitmap.h>
#include <wx/button.h>
#include <wx/imagpng.h>
#include <wx/image.h>
#include <wx/textctrl.h>
#include <wx/colour.h>
#include <wx/panel.h>
#include <wx/dirctrl.h>
#include <wx/listctrl.h>
#include <string>
#include <vector>
#include <algorithm>
#include <wx/progdlg.h>
#include <wx/dirdlg.h>
#include <wx/file.h>
#include <wx/tokenzr.h>
#include <wx/frame.h>
#include <wx/checklst.h>
#include <wx/popupwin.h>
#include <wx/regex.h>
#include <wx/thread.h>
#include <wx/dir.h>
#include <dirent.h>
#include <wx/statbox.h>
#include <wx/checkbox.h>
#include <wx/encconv.h>
#include "Options_Handler.h"
#include "dir_portable.h"
#include "checklistctrl.h"
#include <wx/dynarray.h>
#include <wx/mimetype.h>

#if defined(unix) || defined(__unix) || defined(__unix__)
    const char SEPARATOR='/';
#else
    const char SEPARATOR='\\';
#endif

//La liste des genres ID3v1
const wxString GENRES[]={_T("Blues"),_T("ClassicRock"),_T("Country"),_T("Dance"),_T("Disco"),_T("Funk"),_T("Grunge"),_T("Hip-Hop"),_T("Jazz"),
_T("Metal"),_T("NewAge"),_T("Oldies"),_T("Other"),_T("Pop"),_T("R&B"),_T("Rap"),_T("Reggae"),_T("Rock"),_T("Techno"),_T("Industrial"),_T("Alternative"),
_T("Ska"),_T("DeathMetal"),_T("Pranks"),_T("Soundtrack"),_T("Euro-Techno"),_T("Ambient"),_T("Trip-Hop"),_T("Vocal"),_T("Jazz+Funk"),_T("Fusion"),
_T("Trance"),_T("Classical"),_T("Instrumental"),_T("Acid"),_T("House"),_T("Game"),_T("SoundClip"),_T("Gospel"),_T("Noise"),_T("AlternRock"),_T("Bass"),
_T("Soul"),_T("Punk"),_T("Space"),_T("Meditative"),_T("InstrumentalPop"),_T("InstrumentalRock"),_T("Ethnic"),_T("Gothic"),_T("Darkwave"),
_T("Techno-Industrial"),_T("Electronic"),_T("Pop-Folk"),_T("Eurodance"),_T("Dream"),_T("SouthernRock"),_T("Comedy"),_T("Cult"),_T("Gangsta"),
_T("Top40"),_T("ChristianRap"),_T("Pop/Funk"),_T("Jungle"),_T("NativeAmerican"),_T("Cabaret"),_T("NewWave"),_T("Psychadelic"),_T("Rave"),_T("Showtunes"),
_T("Trailer"),_T("Lo-Fi"),_T("Tribal"),_T("AcidPunk"),_T("AcidJazz"),_T("Polka"),_T("Retro"),_T("Musical"),_T("Rock & Roll"),_T("HardRock"),_T("Folk"),
_T("Folk-Rock"),_T("NationalFolk"),_T("Swing"),_T("FastFusion"),_T("Bebob"),_T("Latin"),_T("Revival"),_T("Celtic"),_T("Bluegrass"),_T("Avantgarde"),
_T("GothicRock"),_T("ProgressiveRock"),_T("PsychedelicRock"),_T("SymphonicRock"),_T("SlowRock"),_T("BigBand"),_T("Chorus"),_T("EasyListening"),
_T("Acoustic"), _T("Humour"),_T("Speech"),_T("Chanson"),_T("Opera"),_T("ChamberMusic"),_T("Sonata"),_T("Symphony"),_T("BootyBass"),_T("Primus"),
_T("PornGroove"),_T("Satire"),_T("SlowJam"),_T("Club"),_T("Tango"),_T("Samba"),_T("Folklore"),_T("Ballad"),_T("PowerBallad"),_T("RhythmicSoul"),_T("Freestyle"),
_T("Duet"),_T("PunkRock"),_T("DrumSolo"),_T("Acapella"),_T("Euro-House"),_T("DanceHall")};


/**
 *Class which copy or move files in his own thread
 */
class CopyMoveThread : public wxThread{
    public:
       /**
        *Constructor
        * @param l the list of files
        * @param b the notebook
        * @param d the directory in which the files will be copied/moved
        * @param move if false just copy, else move files
        */
        CopyMoveThread(wxListCtrl* l,wxNotebook* b,wxString& d,bool move=false);
        virtual void *Entry();
        virtual void OnExit();
        long getProg();
        int getErrors();

    private:
        enum{ID_THREAD=756};
        wxListCtrl* ml;
        wxNotebook* mb;
        bool stop,mv;
        long prog;
        int errors;
        wxString dossier;

};

/**
 *A modal frame
 */
class wxModalFrame : public wxFrame{
    private:
        void onClose(wxCloseEvent& event);
        DECLARE_EVENT_TABLE()
    public:
        wxModalFrame(wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = wxDEFAULT_FRAME_STYLE, const wxString& name = _T("frame"));
};

//defini l'interieur d'un case des listes des comboboxes
struct inTab{
    //Pour les doublons
    wxString label;
    //correspond a son index dans la combobox
    long index;
};

WX_DEFINE_ARRAY(struct inTab *, ComboTab);

class ComboBox : public wxComboBox{
    public:
        ComboTab getList(){return inList;};
        void setList(unsigned int indexList, inTab* uneCase);
        void add(inTab*);
        void del(long);
        wxString getText(long);
        ComboBox(wxWindow* parent, wxWindowID id, const wxString& value,const wxPoint& pos, const wxSize& size, const wxArrayString& choices, long style=0);
    private:
        ComboTab inList;
};

/**
 *The notebook which contains button and combobox to modifie the tags/filenames
 */
class MonBook : public wxNotebook{
    private:
        wxPanel* panelPage1;
        wxPanel* panelPage2;
        wxPanel* panelPage3;
        wxPanel* panelPage4;
        ComboBox* comboTitre;
        ComboBox* comboInterprete;
        ComboBox* comboAlbum;
        ComboBox* comboAnne;
        ComboBox* comboPiste;
        ComboBox* comboGenre;
        wxBitmapButton* button_save;
        ComboBox* comboTitre2;
        ComboBox* comboInterprete2;
        ComboBox* comboAlbum2;
        ComboBox* comboAnne2;
        ComboBox* comboPiste2;
        ComboBox* comboGenre2;
        wxBitmapButton* button_save2;
        wxComboBox* comboOrg;
        wxButton* btnV2ToV1;
        wxButton* btnV1ToV2;
        wxButton* btnCopier;
        wxButton* btnDeplacer;
        //wxButton* btnCleanID3V2;
        wxTextCtrl* DescrSaisie;
        wxStaticText* infoSaisie;
        wxComboBox* comboNomFic;
        wxButton* btnApercu;
        wxTextCtrl* DescrSaisie2;
        wxCheckBox* chkTitre;
        wxCheckBox* chkAlbum;
        wxCheckBox* chkAnne;
        wxCheckBox* chkGenre;
        wxCheckBox* chkPiste;
        wxCheckBox* chkArtiste;
        wxCheckBox* chkNoEmpty;
        wxString text1;
        wxString text2;
        wxArrayString* lstComboTitre;
        wxArrayString* lstComboInterprete;
        wxArrayString* lstComboAlbum;
        wxArrayString* lstComboPiste;
        wxArrayString* lstComboTitre2;
        wxArrayString* lstComboInterprete2;
        wxArrayString* lstComboAlbum2;
        wxArrayString* lstComboPiste2;
        wxButton* btnAnnuler;
        wxButton* btnValider;
        wxCheckListCtrl* LstBox;
        wxModalFrame* containList;
        wxArrayString listeNoms,listeNoms2;
        wxProgressDialog* progBar;
        CopyMoveThread* thread;
        enum{ID_SAVE=637,ID_SAVE2,ID_V1TOV2,ID_V2TOV1,ID_CLEAN,ID_COPIER,ID_DEPLACER,ID_APPERCU,ID_COMBOORG,ID_FIC,ID_TRACK,ID_ANNEE,ID_THREAD=756,ID_ANNULER=1758,ID_VALIDER};
        wxListCtrl* liste;
	wxComboBox* comboFormatPiste;
	wxGenericDirCtrl* arbo;
        void button_click(wxCommandEvent&);
        void button_clickV1(wxCommandEvent&);
        void btnCopyClick(wxCommandEvent&);
        void btnMoveClick(wxCommandEvent&);
        void v1ToV2(wxCommandEvent&);
        void v2ToV1(wxCommandEvent&);
        //void btnCleanID3v2Click(wxCommandEvent& event);
        void btnApercuRename(wxCommandEvent&);
        void btnValiderClick(wxCommandEvent&);
        void btnAnnulerClick(wxCommandEvent&);
        void textWriting(wxCommandEvent&);
        void textWriting2(wxCommandEvent&);
        void textWritingNumbers(wxCommandEvent&);
        void textWritingNumbers2(wxCommandEvent&);
        void cpyMoveThreadMsg(wxCommandEvent&);
        DECLARE_EVENT_TABLE()

    public:
        /**
         * Constructor
         * @param parent the parent of the component
         */
        MonBook(wxWindow *parent);
        void setListe(wxListCtrl* l);
        void setArbo(wxGenericDirCtrl* a);
        //getters
        ComboBox* getComboTitre(){ return comboTitre;}
        ComboBox* getComboInterprete(){ return comboInterprete;}
        ComboBox* getComboAlbum(){ return comboAlbum;}
        ComboBox* getComboAnne(){ return comboAnne;}
        ComboBox* getComboPiste(){ return comboPiste;}
        ComboBox* getComboGenre(){ return comboGenre;}
        ComboBox* getComboTitre2(){ return comboTitre2;}
        ComboBox* getComboInterprete2(){ return comboInterprete2;}
        ComboBox* getComboAlbum2(){ return comboAlbum2;}
        ComboBox* getComboAnne2(){ return comboAnne2;}
        ComboBox* getComboPiste2(){ return comboPiste2;}
        ComboBox* getComboGenre2(){ return comboGenre2;}
        wxComboBox* getComboOrg(){ return comboOrg;}
        wxArrayString* getLstComboTitre(){ return lstComboTitre; }
        wxArrayString* getLstComboInterprete(){ return lstComboInterprete; }
        wxArrayString* getLstComboAlbum(){ return lstComboAlbum; }
        wxArrayString* getLstComboPiste(){ return lstComboPiste;}
        wxArrayString* getLstComboTitre2(){ return lstComboTitre2; }
        wxArrayString* getLstComboInterprete2(){ return lstComboInterprete2; }
        wxArrayString* getLstComboAlbum2(){ return lstComboAlbum2; }
        wxArrayString* getLstComboPiste2(){ return lstComboPiste2;}
        /**
         * Smart removing of non ascii characters
         * @param  s the string to clean
         * @param  replacement the replacement characters
         */
        void cleanStr(wxString&,wxString&);
        void showMsg(wxString,wxString);
        static wxString ucwords(const wxString&);

};

/**
 *The listCtrl to show id3V2 tags of files in current directory
 */
class MaListe : public wxListCtrl{
    private:
        MonBook* book;
        wxGenericDirCtrl* arbo;
        std::vector<std::string> currentFiles;
        wxArrayString* tagsV1[6];
        wxArrayString* tagsV1Sorted[6];
        int currentCol;
        bool desc,sortByFolder,is_key_pressed;
        wxImageList* columnImageList;
        void onContextMenu(wxContextMenuEvent& event);
        void touche(wxKeyEvent&);
        void toucheHaut(wxKeyEvent&);
        void onItemSelected(wxListEvent& event);
        void onItemDeselected(wxListEvent& event);
        void sort(wxListEvent& event);
        void onRefresh(wxCommandEvent& WXUNUSED(event));
        void onDel(wxCommandEvent& WXUNUSED(event));
        void onSel(wxCommandEvent& WXUNUSED(event));
        void playCurrent(wxMouseEvent& e);
        enum{ID_REFRESH=13,ID_DEL,ID_SEL,ID_MSG=7777};
        DECLARE_EVENT_TABLE()
    public:
        MaListe(wxWindow *parent, MonBook* b);

        /**
         * Extract tags from a file and puts them into a new line
         * @param std::string the path to the file
         * @param int the index of the line to fill 
         */
        void readAndShowInfos(std::string f,int index);
        void addToCurrent(std::string);
        void clearCurrent();
        /**
         * Get the text of a specic column in a row
         * @param  long the index of the line
         * @param  int the column
         * @return the text found
         */
        wxString getRowText(long, int);
        /**
         * get the text of a specic column in the selected row
         * @param  int the column
         * @param  int the index of the previous selected row, may be -1 to select the first selected row
         * @return the text found
         */
        wxString getSelRowText(int , int );
        /**
         * get the index of the next selected row
         * @param  int the current index of the row, may be -1 a specic column in
         * @return the next selected row index or -1 if not found
         */
        long getSelection(int);
        void fill_combo();
        void setArbo(wxGenericDirCtrl* a);

        void unblockKeyEvent(bool b=true);
        //getters
        int getCurrentCol(){return currentCol;}
        wxArrayString* getTitreV1(){ return tagsV1[0];}
        wxArrayString* getArtisteV1(){ return tagsV1[1];}
        wxArrayString* getAlbumV1(){ return tagsV1[2];}
        wxArrayString* getAnneeV1(){ return tagsV1[3];}
        wxArrayString* getGenreV1(){ return tagsV1[4];}
        wxArrayString* getPisteV1(){ return tagsV1[5];}
        wxArrayString* getTitreV1Sorted(){ return tagsV1Sorted[0];}
        wxArrayString* getArtisteV1Sorted(){ return tagsV1Sorted[1];}
        wxArrayString* getAlbumV1Sorted(){ return tagsV1Sorted[2];}
        wxArrayString* getAnneeV1Sorted(){ return tagsV1Sorted[3];}
        wxArrayString* getGenreV1Sorted(){ return tagsV1Sorted[4];}
        wxArrayString* getPisteV1Sorted(){ return tagsV1Sorted[5];}

        bool sortAscending(){return !desc;}
        void removeColumnImage();
        /**
         * remove the selected row(s)
         */
        void delCurrent();
        /**
         * Send a MENU_EVT for the main frame to change the state of two edit menu items (enable or not)
         * @param  int the mode : -1 to disable all, 0 to enable all, 1 to enable just the first and 2 to enable just the second
         */
        void sendMenuMsg(int);
        wxArrayString  itemDataMap[9];
        enum{NAME_COL=0,TITLE_COL,ARTIST_COL,ALBUM_COL,YEAR_COL,GENRE_COL,TRACK_COL,LEN_COL,DIR_COL};


};

/**
 *A pop-up window whith a waiting message and a cancel button
 */
class WaitPopup : public wxPopupWindow{
    public:
        WaitPopup(wxGenericDirCtrl*);


    private:
        wxGenericDirCtrl* ma;
        //cancel the current scan
        void Cancel(wxCommandEvent& event);
        enum{ID_ANNULER=42};
        DECLARE_EVENT_TABLE()


};

/**
 *A thread which list the file whith mp3 extension in a directory
 */
class ScanThread : public wxThread{
    public:
       /**
        * The constructor
        * @param a 
        * @param c the absolute path of the directory to scan
        * @param f the list of files fund
        * @param rec if true the scan will be recursive
        */
        ScanThread(wxGenericDirCtrl* a,std::string c,std::vector<std::string>* f, bool rec=true);
       /**
        * Scan recusirvely a directory and store path of mp3's file found
        * @param chemin the root path
        * @param r if true the scan will be recursive
        */
        void walkThrough(std::string chemin,bool r);
        virtual void *Entry();
        virtual void OnExit();
        /**
         *Stop immediately the scanning
         */
        void Stop();

    private:
        enum{THREAD=1234};
        wxGenericDirCtrl* ma;
        bool stop,recursive;
        std::string chemin;
        std::vector<std::string>* file;

};

/**
 *A list of directory on the computer
 */
class MonArbo : public wxGenericDirCtrl{
    private:
        enum{THREAD=1234};
        MaListe* liste;
        MonBook* book;
        WaitPopup* wait;
        ScanThread* t;
        std::vector<std::string> files;
        bool ignore_event;
        void change(wxTreeEvent& e);
        //this function is called when the ScanThread has done its task
        void endChange(wxCommandEvent& e);
        DECLARE_DYNAMIC_CLASS(MonArbo)
        DECLARE_EVENT_TABLE()
    public:
        /**
         *The constructor
         * @param wxWindow* the parent of the component (may be NULL)
         * @param MaListe* the wxListCtrl
         * @param MonBook* the wxNotebook
         */
        MonArbo(wxWindow *parent,MaListe* l,MonBook* b);
        MonArbo(){}//for the dynmic class
        void stopThread();

        /**
         * Define if the class will do something when an EVT_TREE_SEL_CHANGED is received
         * @param bool if true the class will ignore the event
         */
	void setIgnore(bool);
        void refresh();

};

class ListCtrlPanel : public wxPanel{
    public:
        ListCtrlPanel(wxWindow *parent,MonBook* b);
        MaListe* getMListe();
    private:
        MaListe* mliste;

};

#endif // COMPONENTS_H_INCLUDED
